#ifdef __cplusplus
extern "C" {
#endif

#ifndef CHorde3D
#define CHorde3D

typedef unsigned int Horde3D_ResHandle;
typedef unsigned int Horde3D_NodeHandle;

const Horde3D_NodeHandle Horde3D_RootNode = 1;
const Horde3D_NodeHandle Horde3D_PrimeTimeCam = 2;

typedef enum
{
	NoQuery = 1,
	NoTexPOTConversion = 2,
	NoTexCompression = 4,
	NoTexMipmaps = 8,
	NoTexFiltering = 16,
	NoTexRepeat = 32
} Horde3D_ResourceFlags;

typedef enum
{
	TrilinearFiltering,
	AnisotropyFactor,
	TexCompression,
	LoadTextures,
	FastAnimation,
	OcclusionCulling,
	ShadowMapSize,
	DebugViewMode
} Horde3D_EngineOptions;

typedef enum
{
	SceneGraph,
	Geometry,
	Animation,
	Material,
	Code,
	Shader,
	Texture2D,
	TextureCube,
	Effect,
	UndefinedResourceType = 9999
} Horde3D_ResourceTypes;

typedef enum
{
	AnimFrameCount,
	Tex2DPixelData
} Horde3D_ResourceData;

typedef enum
{
	Group,
	Model,
	Mesh,
	Joint,
	Light,
	Camera,
	Emitter,
	UndefinedSceneNodeType = 9999
} Horde3D_SceneNodeTypes;

typedef enum
{
	MinDist,
	MaxDist
} Horde3D_GroupNodeParams;

typedef enum
{
	MeshMaterialRes
} Horde3D_MeshNodeParams;

typedef enum
{
	LightMaterialRes,
	Radius,
	FOV,
	Col_R,
	Col_G,
	Col_B,
	ShadowMapCount,
	ShadowSplitLambda,
	ShadowMapBias
} Horde3D_LightNodeParams;

typedef enum
{
	LeftPlane,
	RightPlane,
	BottomPlane,
	TopPlane,
	NearPlane,
	FarPlane
} Horde3D_CameraNodeParams;

typedef enum
{
	MaterialRes,
	EffectRes,
	MaxCount,
	RespawnCount,
	Delay,
	EmissionRate,
	SpreadAngle,
	ForceX,
	ForceY,
	ForceZ
} Horde3D_EmitterNodeParams;

#endif

	// Basic
extern const char *Horde3D_getVersionString();
extern int Horde3D_init();
extern void Horde3D_release();
extern void Horde3D_resize( int x, int y, int width, int height );
extern void Horde3D_render();
extern void Horde3D_clear();

	// Pipeline
extern int Horde3D_loadPipelineConfig( const char *configFilename );

	// Messages
extern const char *Horde3D_getMessage( unsigned int *level, float *time );

	// Active Camera
extern Horde3D_NodeHandle Horde3D_getActiveCamera();
extern int Horde3D_setActiveCamera( Horde3D_NodeHandle cameraNode );

	// Options
extern float Horde3D_getOption( Horde3D_EngineOptions param );
extern int Horde3D_setOption( Horde3D_EngineOptions param, float value );

	// Overlay
extern void Horde3D_showOverlay( float x_ll, float y_ll, float u_ll, float v_ll,
						  float x_lr, float y_lr, float u_lr, float v_lr,
						  float x_ur, float y_ur, float u_ur, float v_ur,
						  float x_ul, float y_ul, float u_ul, float v_ul,
						  unsigned int layer, Horde3D_ResHandle materialRes );

	// Render Target
extern int Horde3D_getRenderTargetData( const char *targetName, unsigned int bufIndex,
						  unsigned int *width, unsigned int *height, unsigned int *compCount,
						  float *dataBuffer, unsigned int bufferSize );

	// Resources
extern Horde3D_ResourceTypes Horde3D_getResourceType( Horde3D_ResHandle res );
extern Horde3D_ResHandle Horde3D_findResource( Horde3D_ResourceTypes type, const char *name );
extern Horde3D_ResHandle Horde3D_addResource( Horde3D_ResourceTypes type, const char *name, unsigned int flags );
extern int Horde3D_removeResource( Horde3D_ResHandle res );
extern int Horde3D_loadResource( const char *name, const char *data, unsigned int size );
extern int Horde3D_unloadResource( Horde3D_ResHandle res );
extern const void *Horde3D_getResourceData( Horde3D_ResHandle res, Horde3D_ResourceData param );
extern int Horde3D_updateResourceData( Horde3D_ResHandle res, Horde3D_ResourceData param, const void *data, unsigned int size );
extern const char *Horde3D_queryUnloadedResource();
extern const char *Horde3D_getResourcePath( Horde3D_ResourceTypes type );
extern void Horde3D_setResourcePath( Horde3D_ResourceTypes type, const char *path );
extern void Horde3D_releaseUnusedResources();
extern int Horde3D_setMaterialUniform( Horde3D_ResHandle materialRes, const char *name, float a, float b, float c, float d );

	// Nodes
extern Horde3D_SceneNodeTypes Horde3D_getNodeType( Horde3D_NodeHandle node );
extern const char *Horde3D_getNodeName( Horde3D_NodeHandle node );
extern int Horde3D_setNodeName( Horde3D_NodeHandle node, const char *name );
extern Horde3D_NodeHandle Horde3D_getNodeParent( Horde3D_NodeHandle node );
extern Horde3D_NodeHandle Horde3D_getNodeChild( Horde3D_NodeHandle node, unsigned int index );
extern Horde3D_NodeHandle Horde3D_addNodes( Horde3D_NodeHandle parent, Horde3D_ResHandle sceneGraphRes );
extern int Horde3D_removeNode( Horde3D_NodeHandle node );
extern int Horde3D_setNodeActivation( Horde3D_NodeHandle node, int active );
extern int Horde3D_getNodeTransform( Horde3D_NodeHandle node, float *px, float *py, float *pz,
							   float *rx, float *ry, float *rz, float *sx, float *sy, float *sz );
extern int Horde3D_setNodeTransform( Horde3D_NodeHandle node, float px, float py, float pz,
							   float rx, float ry, float rz, float sx, float sy, float sz );
extern int Horde3D_getNodeTransformMatrices( Horde3D_NodeHandle node, const float **relMat, const float **absMat );
extern int Horde3D_setNodeTransformMatrix( Horde3D_NodeHandle node, const float *mat4x4 );
extern int Horde3D_getNodeAABB( Horde3D_NodeHandle node, float *minX, float *minY, float *minZ,
						  float *maxX, float *maxY, float *maxZ );
extern unsigned int Horde3D_findNodes( Horde3D_NodeHandle startNode, const char *name, Horde3D_SceneNodeTypes type );
extern Horde3D_NodeHandle Horde3D_getNodeFindResult( unsigned int index );
extern Horde3D_NodeHandle Horde3D_castRay( Horde3D_NodeHandle node, float ox, float oy, float oz, float dx, float dy, float dz );

	// Group Node
extern Horde3D_NodeHandle Horde3D_addGroupNode( Horde3D_NodeHandle parent, const char *name );
extern float Horde3D_getGroupParam( Horde3D_NodeHandle node, Horde3D_GroupNodeParams param );
extern int Horde3D_setGroupParam( Horde3D_NodeHandle node, Horde3D_GroupNodeParams param, float value );

	// Model Node
extern Horde3D_NodeHandle Horde3D_addModelNode( Horde3D_NodeHandle parent, const char *name, Horde3D_ResHandle geometryRes );
extern int Horde3D_setupModelAnimStage( Horde3D_NodeHandle node, unsigned int stage, Horde3D_ResHandle animationRes,
								  const char *startNode, int additive );
extern int Horde3D_setModelAnimParams( Horde3D_NodeHandle node, unsigned int stage, float time, float weight );
extern int Horde3D_setModelMorpher( Horde3D_NodeHandle node, const char *target, float weight );

	// Mesh Node
extern Horde3D_NodeHandle Horde3D_addMeshNode( Horde3D_NodeHandle parent, const char *name, Horde3D_ResHandle materialRes, 
								unsigned int batchStart, unsigned int batchCount,
								unsigned int vertRStart, unsigned int vertREnd );
extern float Horde3D_getMeshParam( Horde3D_NodeHandle node, Horde3D_MeshNodeParams param );
extern int Horde3D_setMeshParam( Horde3D_NodeHandle node, Horde3D_MeshNodeParams param, float value );
extern int Horde3D_getMeshData( Horde3D_NodeHandle node, unsigned int *vertPosArrayCount, const float **vertPosArrayData,
					  unsigned int *indexArrayCount, const unsigned int **indexArayData, unsigned int *indexOffset );
extern int Horde3D_setMeshUniform( Horde3D_NodeHandle node, const char *name, float a, float b, float c, float d, int recursive );

	// Joint Node
extern Horde3D_NodeHandle Horde3D_addJointNode( Horde3D_NodeHandle parent, const char *name, unsigned int jointIndex );

	// Light Node
extern Horde3D_NodeHandle Horde3D_addLightNode( Horde3D_NodeHandle parent, const char *name, Horde3D_ResHandle materialRes,
								 const char *lightingContext, const char *shadowContext );
extern float Horde3D_getLightParam( Horde3D_NodeHandle node, Horde3D_LightNodeParams param );
extern int Horde3D_setLightParam( Horde3D_NodeHandle node, Horde3D_LightNodeParams param, float value );
extern int Horde3D_setLightContexts( Horde3D_NodeHandle node, const char *lightingContext, const char *shadowContext );

	// Camera Node
extern Horde3D_NodeHandle Horde3D_addCameraNode( Horde3D_NodeHandle parent, const char *name );
extern float Horde3D_getCameraParam( Horde3D_NodeHandle node, Horde3D_CameraNodeParams param );
extern int Horde3D_setCameraParam( Horde3D_NodeHandle node, Horde3D_CameraNodeParams param, float value );
extern int Horde3D_setupCameraView( Horde3D_NodeHandle node, float fov, float aspect, float nearDist, float farDist );
extern int Horde3D_calcCameraProjectionMatrix( Horde3D_NodeHandle node, float *projMat );

	// Emitter Node
extern Horde3D_NodeHandle Horde3D_addEmitterNode( Horde3D_NodeHandle parent, const char *name,
								   Horde3D_ResHandle materialRes, Horde3D_ResHandle effectRes,
								   unsigned int maxParticleCount, int respawnCount );
extern float Horde3D_getEmitterParam( Horde3D_NodeHandle node, Horde3D_EmitterNodeParams param );
extern int Horde3D_setEmitterParam( Horde3D_NodeHandle node, Horde3D_EmitterNodeParams param, float value );
extern int Horde3D_advanceEmitterTime( Horde3D_NodeHandle node, float timeDelta );

#ifdef __cplusplus
}
#endif

